/* GStreamer
 *
 * Copyright (C) 2003 Ronald Bultje <rbultje@ronald.bitfreak.net>
 *               2006 Edgard Lima <edgard.lima@indt.org.br>
 *               2008-2010 Nokia Corporation <multimedia@maemo.org>
 *
 * gstcameracolorbalance.c: generic color balance interface implementation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gst/gst.h>
#include "gstcameracolorbalance.h"

GST_BOILERPLATE (GstCameraSrcColorBalanceChannel,
    gst_camerasrc_color_balance_channel,
    GstColorBalanceChannel, GST_TYPE_COLOR_BALANCE_CHANNEL);

static void
gst_camerasrc_color_balance_channel_base_init (gpointer g_class)
{
}

static void
gst_camerasrc_color_balance_channel_class_init (GstCameraSrcColorBalanceChannelClass *
    klass)
{
}

static void
gst_camerasrc_color_balance_channel_init (GstCameraSrcColorBalanceChannel * channel,
    GstCameraSrcColorBalanceChannelClass * klass)
{
  channel->id = (guint32) - 1;
}

static G_GNUC_UNUSED gboolean
gst_camerasrc_color_balance_contains_channel (GstCameraSrc * camerasrc,
    GstCameraSrcColorBalanceChannel * channel)
{
  const GList *item;

  for (item = camerasrc->colors; item != NULL; item = item->next)
    if (item->data == channel)
      return TRUE;

  return FALSE;
}

const GList *
gst_camerasrc_color_balance_list_channels (GstCameraSrc * camerasrc)
{
  return camerasrc->colors;
}

void
gst_camerasrc_color_balance_set_value (GstCameraSrc * camerasrc,
    GstColorBalanceChannel * channel, gint value)
{
  GstCameraSrcClass *bclass;
  gboolean opened;

  GstCameraSrcColorBalanceChannel *camchannel =
      GST_CAMERA_SRC_COLOR_BALANCE_CHANNEL (channel);

  bclass = GST_CAMERA_SRC_GET_CLASS (camerasrc);
  opened = bclass->is_open (camerasrc);

  /* assert that we're opened and that we're using a known item */
  g_return_if_fail (opened);
  g_return_if_fail (gst_camerasrc_color_balance_contains_channel (camerasrc,
          camchannel));

  bclass->set_attribute (camerasrc, camchannel->id, value);
}

gint
gst_camerasrc_color_balance_get_value (GstCameraSrc * camerasrc,
    GstColorBalanceChannel * channel)
{
  GstCameraSrcClass *bclass;
  gboolean opened;

  GstCameraSrcColorBalanceChannel *camchannel =
      GST_CAMERA_SRC_COLOR_BALANCE_CHANNEL (channel);
  gint value;

  bclass = GST_CAMERA_SRC_GET_CLASS (camerasrc);
  opened = bclass->is_open (camerasrc);

  /* assert that we're opened and that we're using a known item */
  g_return_val_if_fail (opened, 0);
  g_return_val_if_fail (gst_camerasrc_color_balance_contains_channel (camerasrc,
          camchannel), 0);

  if (!bclass->get_attribute (camerasrc, camchannel->id, &value))
    return 0;

  return value;
}
