/* GStreamer
 *
 * Copyright (C) 2008-2010 Nokia Corporation <multimedia@maemo.org>
 *
 * gstcameraphotoiface.h: Photo interface implementation for camerasrc
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GST_CAMERA_SRC_PHOTOGRAPHY_H__
#define __GST_CAMERA_SRC_PHOTOGRAPHY_H__

#include <gst/gst.h>
#include <gst/interfaces/photography.h>

#include "gstcamerasrc.h"

G_BEGIN_DECLS

typedef enum {
  PROP_0,
  PROP_ALWAYS_COPY,
  PROP_CAPTURE_MODE,
  PROP_VIEWFINDER_MODE,
  PROP_DRIVER_NAME,
  PROP_WB_MODE,
  PROP_COLOUR_TONE,
  PROP_SCENE_MODE,
  PROP_FLASH_MODE,
  PROP_CAPABILITIES,
  PROP_EV_COMP,
  PROP_ISO_SPEED,
  PROP_APERTURE,
  PROP_EXPOSURE,
  PROP_ZOOM,
  PROP_FLICKER_MODE,
  PROP_FOCUS_MODE,
  PROP_IMAGE_CAPTURE_CAPS,
  PROP_IMAGE_PREVIEW_CAPS,
  PROP_AUTOFOCUS,
  PROP_NOISE_REDUCTION
} GstCameraSrcProperties;


gboolean
gst_camerasrc_photo_set_ev_compensation (GstCameraSrc * camerasrc,
                                          gfloat ev_comp);

gboolean
gst_camerasrc_photo_get_ev_compensation (GstCameraSrc * camerasrc,
                                          gfloat *ev_comp);

gboolean
gst_camerasrc_photo_set_iso_speed (GstCameraSrc * camerasrc,
                                    guint iso_speed);

gboolean
gst_camerasrc_photo_get_iso_speed (GstCameraSrc * camerasrc,
                                    guint *iso_speed);

gboolean
gst_camerasrc_photo_set_aperture (GstCameraSrc * camerasrc,
                                   guint aperture);

gboolean
gst_camerasrc_photo_get_aperture (GstCameraSrc * camerasrc,
                                   guint *aperture);

gboolean
gst_camerasrc_photo_set_exposure (GstCameraSrc * camerasrc,
                                   guint32 exposure);

gboolean
gst_camerasrc_photo_get_exposure (GstCameraSrc * camerasrc,
                                   guint32 *exposure);

gboolean
gst_camerasrc_photo_set_wb_mode (GstCameraSrc * camerasrc,
                                  GstWhiteBalanceMode mode);

gboolean
gst_camerasrc_photo_get_wb_mode (GstCameraSrc * camerasrc,
                                  GstWhiteBalanceMode *mode);

gboolean
gst_camerasrc_photo_set_tone_mode (GstCameraSrc * camerasrc,
                                    GstColourToneMode mode);

gboolean
gst_camerasrc_photo_get_tone_mode (GstCameraSrc * camerasrc,
                                    GstColourToneMode *mode);

gboolean
gst_camerasrc_photo_set_scene_mode (GstCameraSrc * camerasrc,
                                     GstSceneMode mode);

gboolean
gst_camerasrc_photo_get_scene_mode (GstCameraSrc * camerasrc,
                                     GstSceneMode *mode);

gboolean
gst_camerasrc_photo_set_flash_mode (GstCameraSrc * camerasrc,
                                     GstFlashMode mode);

gboolean
gst_camerasrc_photo_get_flash_mode (GstCameraSrc * camerasrc,
                                     GstFlashMode *mode);

gboolean
gst_camerasrc_photo_set_zoom (GstCameraSrc * camerasrc,
                               gfloat zoom);

gboolean
gst_camerasrc_photo_get_zoom (GstCameraSrc * camerasrc,
                               gfloat *zoom);

gboolean
gst_camerasrc_photo_set_flicker_mode (GstCameraSrc * camerasrc,
                               GstFlickerReductionMode mode);

gboolean
gst_camerasrc_photo_get_flicker_mode (GstCameraSrc * camerasrc,
                               GstFlickerReductionMode *mode);

gboolean
gst_camerasrc_photo_set_focus_mode (GstCameraSrc * camerasrc,
                               GstFocusMode mode);
gboolean
gst_camerasrc_photo_get_focus_mode (GstCameraSrc * camerasrc,
                               GstFocusMode *mode);

gboolean
gst_camerasrc_photo_set_noise_reduction (GstCameraSrc * camerasrc,
                                         GstPhotographyNoiseReduction mode);
gboolean
gst_camerasrc_photo_get_noise_reduction (GstCameraSrc * camerasrc,
                                         GstPhotographyNoiseReduction *mode);

void
gst_camerasrc_photo_set_autofocus (GstCameraSrc * camerasrc,
                                    gboolean on);

gboolean
gst_camerasrc_photo_prepare_for_capture (GstCameraSrc * camerasrc,
                                    GstPhotoCapturePrepared func,
                                    GstCaps *capture_caps,
                                    gpointer user_data);

GstPhotoCaps
gst_camerasrc_photo_get_capabilities (GstCameraSrc * camerasrc);

void
gst_camerasrc_photo_ready_for_capture (GstCameraSrc *camerasrc,
                                        GstCaps *selected);

gboolean
gst_camerasrc_photo_set_config (GstCameraSrc * camerasrc,
                                 GstPhotoSettings * config);

gboolean
gst_camerasrc_photo_get_config (GstCameraSrc * camerasrc,
                                 GstPhotoSettings * config);

gboolean
gst_camerasrc_photo_set_format (GstCameraSrc * camerasrc,
                                GstOperationMode op_mode,
                                GstCaps * op_mode_caps);
GstCaps *
gst_camerasrc_photo_get_format (GstCameraSrc * camerasrc,
                                GstOperationMode op_mode);

gboolean
gst_camerasrc_photo_set_property (GstCameraSrc *camerasrc,
                             guint prop_id,
                             const GValue * value);

gboolean
gst_camerasrc_photo_get_property (GstCameraSrc *camerasrc,
                             guint prop_id,
                             GValue * value);

#define GST_CAMERA_SRC_PHOTO_FUNCS(Type, interface_as_function, function_name, param_type) \
                                                                                      \
gboolean                                                                              \
interface_as_function ## _photo_set_ ## function_name (GstPhotography * photo,        \
                                                       param_type param)              \
{                                                                                     \
  Type *this = (Type*) photo;                                                         \
  return gst_camerasrc_photo_set_ ## function_name (this, param);                     \
}                                                                                     \
                                                                                      \
gboolean                                                                              \
interface_as_function ## _photo_get_ ## function_name (GstPhotography * photo,        \
                                                       param_type * param)            \
{                                                                                     \
  Type *this = (Type*) photo;                                                         \
  return gst_camerasrc_photo_get_ ## function_name (this, param);                     \
}


#define GST_IMPLEMENT_CAMERA_SRC_PHOTO_METHODS(Type, interface_as_function)           \
                                                                                      \
GST_CAMERA_SRC_PHOTO_FUNCS(Type, interface_as_function, ev_compensation, gfloat)      \
GST_CAMERA_SRC_PHOTO_FUNCS(Type, interface_as_function, iso_speed, guint)             \
GST_CAMERA_SRC_PHOTO_FUNCS(Type, interface_as_function, aperture, guint)              \
GST_CAMERA_SRC_PHOTO_FUNCS(Type, interface_as_function, exposure, guint32)            \
GST_CAMERA_SRC_PHOTO_FUNCS(Type, interface_as_function, wb_mode, GstWhiteBalanceMode) \
GST_CAMERA_SRC_PHOTO_FUNCS(Type, interface_as_function, tone_mode, GstColourToneMode) \
GST_CAMERA_SRC_PHOTO_FUNCS(Type, interface_as_function, scene_mode, GstSceneMode)     \
GST_CAMERA_SRC_PHOTO_FUNCS(Type, interface_as_function, flash_mode, GstFlashMode)     \
GST_CAMERA_SRC_PHOTO_FUNCS(Type, interface_as_function, zoom, gfloat)                 \
GST_CAMERA_SRC_PHOTO_FUNCS(Type, interface_as_function, flicker_mode, GstFlickerReductionMode) \
GST_CAMERA_SRC_PHOTO_FUNCS(Type, interface_as_function, focus_mode, GstFocusMode)     \
GST_CAMERA_SRC_PHOTO_FUNCS(Type, interface_as_function, noise_reduction, GstPhotographyNoiseReduction) \
                                                                                      \
GstPhotoCaps                                                                          \
interface_as_function ## _photo_get_capabilities (GstPhotography * photo)             \
{                                                                                     \
  Type *this = (Type*) photo;                                                         \
  return gst_camerasrc_photo_get_capabilities (this);                                 \
}                                                                                     \
                                                                                      \
gboolean                                                                              \
interface_as_function ## _photo_prepare_for_capture (GstPhotography * photo,          \
                                                     GstPhotoCapturePrepared func,    \
                                                     GstCaps *capture_caps,           \
                                                     gpointer user_data)              \
{                                                                                     \
  Type *this = (Type*) photo;                                                         \
  return gst_camerasrc_photo_prepare_for_capture (this, func,                         \
                                                   capture_caps, user_data);          \
}                                                                                     \
                                                                                      \
void                                                                                  \
interface_as_function ## _photo_set_autofocus (GstPhotography * photo,                \
                                               gboolean on)                           \
{                                                                                     \
  Type *this = (Type*) photo;                                                         \
  gst_camerasrc_photo_set_autofocus (this, on);                                       \
}                                                                                     \
                                                                                      \
gboolean                                                                              \
interface_as_function ## _photo_set_config (GstPhotography * photo,                   \
                                            GstPhotoSettings * config)                \
{                                                                                     \
  Type *this = (Type*) photo;                                                         \
  return gst_camerasrc_photo_set_config (this, config);                               \
}                                                                                     \
                                                                                      \
gboolean                                                                              \
interface_as_function ## _photo_get_config (GstPhotography * photo,                   \
                                            GstPhotoSettings * config)                \
{                                                                                     \
  Type *this = (Type*) photo;                                                         \
  return gst_camerasrc_photo_get_config (this, config);                               \
}                                                                                     \
                                                                                      \
gboolean                                                                              \
interface_as_function ## _photo_set_format (GstPhotography * photo,                   \
                                            GstOperationMode op_mode,                 \
                                            GstCaps * op_mode_caps)                   \
{                                                                                     \
  Type *this = (Type*) photo;                                                         \
  return gst_camerasrc_photo_set_format (this, op_mode, op_mode_caps);                \
}                                                                                     \
                                                                                      \
GstCaps *                                                                             \
interface_as_function ## _photo_get_format (GstPhotography * photo,                   \
                                            GstOperationMode op_mode)                 \
{                                                                                     \
  Type *this = (Type*) photo;                                                         \
  return gst_camerasrc_photo_get_format (this, op_mode);                              \
}                                                                                     \
                                                                                      \
void                                                                                  \
interface_as_function ## _photo_interface_init (GstPhotographyInterface * iface)      \
{                                                                                     \
  iface->set_ev_compensation = interface_as_function ## _photo_set_ev_compensation;   \
  iface->get_ev_compensation = interface_as_function ## _photo_get_ev_compensation;   \
  iface->set_iso_speed = interface_as_function ## _photo_set_iso_speed;               \
  iface->get_iso_speed = interface_as_function ## _photo_get_iso_speed;               \
  iface->set_aperture = interface_as_function ## _photo_set_aperture;                 \
  iface->get_aperture = interface_as_function ## _photo_get_aperture;                 \
  iface->set_exposure = interface_as_function ## _photo_set_exposure;                 \
  iface->get_exposure = interface_as_function ## _photo_get_exposure;                 \
  iface->set_white_balance_mode = interface_as_function ## _photo_set_wb_mode;        \
  iface->get_white_balance_mode = interface_as_function ## _photo_get_wb_mode;        \
  iface->set_colour_tone_mode = interface_as_function ## _photo_set_tone_mode;        \
  iface->get_colour_tone_mode = interface_as_function ## _photo_get_tone_mode;        \
  iface->set_scene_mode = interface_as_function ## _photo_set_scene_mode;             \
  iface->get_scene_mode = interface_as_function ## _photo_get_scene_mode;             \
  iface->set_flash_mode = interface_as_function ## _photo_set_flash_mode;             \
  iface->get_flash_mode = interface_as_function ## _photo_get_flash_mode;             \
  iface->set_zoom = interface_as_function ## _photo_set_zoom;                         \
  iface->get_zoom = interface_as_function ## _photo_get_zoom;                         \
  iface->set_flicker_mode = interface_as_function ## _photo_set_flicker_mode;         \
  iface->get_flicker_mode = interface_as_function ## _photo_get_flicker_mode;         \
  iface->set_focus_mode = interface_as_function ## _photo_set_focus_mode;             \
  iface->get_focus_mode = interface_as_function ## _photo_get_focus_mode;             \
  iface->set_config = interface_as_function ## _photo_set_config;                     \
  iface->get_config = interface_as_function ## _photo_get_config;                     \
  iface->set_format = interface_as_function ## _photo_set_format;                     \
  iface->get_format = interface_as_function ## _photo_get_format;                     \
  iface->set_noise_reduction = interface_as_function ## _photo_set_noise_reduction;   \
  iface->get_noise_reduction = interface_as_function ## _photo_get_noise_reduction;   \
                                                                                      \
  iface->get_capabilities = interface_as_function ## _photo_get_capabilities;         \
  iface->prepare_for_capture = interface_as_function ## _photo_prepare_for_capture;   \
  iface->set_autofocus = interface_as_function ## _photo_set_autofocus;               \
}                                                                                     \


#endif /* __GST_CAMERA_SRC_PHOTOGRAPHY_H__ */
