/* GStreamer
 *
 * Copyright (C) 2001-2002 Ronald Bultje <rbultje@ronald.bitfreak.net>
 *               2006 Edgard Lima <edgard.lima@indt.org.br>
 *               2009 Texas Instruments, Inc - http://www.ti.com/
 *
 * bufferpool.h V4L2 buffer pool class
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GSTSUBDEVBUFFER_H__
#define __GSTSUBDEVBUFFER_H__

#include <linux/videodev2.h>
#include <gst/gst.h>

GST_DEBUG_CATEGORY_EXTERN (subdevsrc_debug);

G_BEGIN_DECLS

#define GST_TYPE_SUBDEVSRC_BUFFER (gst_subdevsrc_buffer_get_type())
#define GST_IS_SUBDEVSRC_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_SUBDEVSRC_BUFFER))
#define GST_SUBDEVSRC_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_SUBDEVSRC_BUFFER, GstSubdevBuffer))

typedef struct _GstSubdevBuffer GstSubdevBuffer;
typedef struct _GstSubdevSrcBufferPool GstSubdevSrcBufferPool;

/**
 * GstSubdevBuffer:
 * @buffer: parent #GstBuffer.
 * @pool: reference to the owning #GstSubdevSrcBufferPool.
 * @gbuffer: #GstBuffer allocated for this buffer for holding the data.
 * @vbuffer: v4l2_buffer structure.
 *
 * Opaque #GstSubdevBuffer object.
 */
struct _GstSubdevBuffer {
  GstBuffer   buffer;
  GstSubdevSrcBufferPool *pool;
  GstBuffer *gbuffer;
  struct v4l2_buffer vbuffer;
};


/**
 * GstSubdevSrcBufferPool:
 * @parent: the parent object
 * @lock: pool lock
 * @running: TRUE if the pool is being used
 * @flushing: TRUE if pool should temporarily ignore buffer handling
 * @num_live_buffers: number of buffers being pushed outside element
 * @buffer_count: total number of buffers used
 * @buffers: buffer table
 * @video_fd: video device which produces the buffers
 * @queued: lookup table for buffers whether they're queued to driver
 * @data_cond: condition being used for waiting a buffer to become available
 *
 * Opague #GstSubdevSrcBufferPool object.
 */
struct _GstSubdevSrcBufferPool
{
  GstMiniObject parent;

  GMutex *lock;
  gboolean running;          /* with lock */
  gboolean flushing;         /* with lock */
  gint num_live_buffers;     /* number of buffers not with driver */
  guint buffer_count;
  GstSubdevBuffer **buffers;

  gint video_fd;             /* a dup(2) of the v4l2object's video_fd */
  guint *queued;
  GCond* data_cond;
};

GstSubdevSrcBufferPool *
gst_subdevsrc_buffer_pool_new      (GstElement * src,
                                    gint fd, GstCaps * caps);

void
gst_subdevsrc_buffer_pool_destroy  (GstSubdevSrcBufferPool * pool,
                                    GstElement * src);

gboolean
gst_subdevsrc_buffer_pool_activate (GstSubdevSrcBufferPool * pool,
                                    GstElement * src);

gboolean
gst_subdevsrc_buffer_pool_update   (GstSubdevSrcBufferPool * pool);

GType gst_subdevsrc_buffer_get_type (void);

G_END_DECLS

#endif /* __GSTSUBDEVBUFFER_H__ */

