/* GStreamer MediaController camera source
 * Copyright (C) 2010 Nokia Corporation <multimedia@maemo.org>
 *
 * Contact: Maemo Multimedia <multimedia@maemo.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GST_CONFIGPARSER_H__
#define __GST_CONFIGPARSER_H__

#include <glib.h>

G_BEGIN_DECLS

typedef struct _GstConfigParser GstConfigParser;
typedef struct _GstConfigAssignment GstConfigAssignment;
typedef struct _GstPipeConfigurationEntry GstPipeConfigurationEntry;
typedef struct _GstPipeConfiguration GstPipeConfiguration;

/**
* GstPipeConfigEntryType:
*
* Enumeration for camera devices.
*/
typedef enum {
  GST_PIPE_ENTRY_ENTITY,
  GST_PIPE_ENTRY_CHANGE_PIXFMT,
  GST_PIPE_ENTRY_SET_OUT_PIXFMT,
  GST_PIPE_ENTRY_SET_OUT_RESOLUTION,
} GstPipeConfigEntryType;


/**
 * GstConfigAssignment:
 *
 * Contains a key-value pair.
 */
struct _GstConfigAssignment {
  gchar *key;
  gchar *value;
};


/**
 * GstPipeConfigurationEntry:
 *
 * Contains information about one pipeline configuration element.
 */
struct _GstPipeConfigurationEntry {
  GstPipeConfigEntryType type;
  union {
    gchar *entity_name;
    guint32 pcode;
  } field;
};


/**
 * GstPipeConfiguration:
 *
 * Contains the whole pipeline configuration structure.
 */
struct _GstPipeConfiguration {
  GSList *entries;
};


/**
 * GstConfigParser:
 *
 * Contains all pipeline configuration data.
 */
struct _GstConfigParser {
  GSList *assignments;
  GstPipeConfiguration *primary;
  GstPipeConfiguration *secondary;
};


GstConfigParser * gst_subdevsrc_config_parser_new (const gchar *location);

void gst_subdevsrc_config_parser_destroy (GstConfigParser *cp);

GSList * gst_subdevsrc_config_parser_get_mappings (GstConfigParser *cp,
    const gchar *key);

G_END_DECLS

#endif /* __GST_CONFIGPARSER_H__ */