/* GStreamer
 *
 * Copyright (C) 2009,2010 Nokia Corporation <multimedia@maemo.org>
 *
 * libomap3camd_calls.h: camera library calls
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __LIBOMAP3CAMD_CALLS__
#define __LIBOMAP3CAMD_CALLS__

#include <gst/gst.h>
#include <gst/interfaces/photography.h>

#include "gstsubdevcamsrc.h"
#include "recyclepool.h"

G_BEGIN_DECLS

/**
 * NotifyCallback:
 * @user_data: user data pointer.
 *
 * A type used for libomap3camd algorithm status change callbacks.
 */
typedef void (*AlgoNotifyCallback) (void *user_data);

gboolean
gst_subdevsrc_libomap3camd_init (GstSubdevSrc *subdevsrc);

void
gst_subdevsrc_libomap3camd_deinit (GstSubdevSrc *subdevsrc);

gboolean
gst_subdevsrc_libomap3camd_handle_roi_event (GstSubdevSrc *subdevsrc,
    const GstStructure *structure);

gboolean
gst_subdevsrc_libomap3camd_handle_motion (GstSubdevSrc *subdevsrc,
    gfloat motionvector, gint64 timedelta);

gboolean
gst_subdevsrc_libomap3camd_daemon_start (GstSubdevSrc *subdevsrc);

gboolean
gst_subdevsrc_libomap3camd_daemon_stop (GstSubdevSrc *subdevsrc);

gboolean
gst_subdevsrc_libomap3camd_grab_hq_frame (GstSubdevSrc *subdevsrc,
    GstBuffer **buf);

gboolean
gst_subdevsrc_libomap3camd_get_makernote (GstSubdevSrc *subdevsrc,
    guint8 **maker_note, guint *maker_note_size);

gboolean
gst_subdevsrc_libomap3camd_get_pp_params (GstSubdevSrc *subdevsrc,
    GstStructure **pp_struct);

gboolean
gst_subdevsrc_libomap3camd_check_shaking (GstSubdevSrc *subdevsrc,
    GstPhotoShakeRisk *risk);

gboolean
gst_subdevsrc_libomap3camd_flash_status (GstSubdevSrc *subdevsrc,
    GstCameraFlashStatus *status);

gboolean
gst_subdevsrc_libomap3camd_flicker_status (GstSubdevSrc *subdevsrc,
    GstFlickerReductionMode *mode);

gboolean
gst_subdevsrc_libomap3camd_set_zoom (GstSubdevSrc *subdevsrc, gfloat zoom);

gboolean
gst_subdevsrc_libomap3camd_get_focus_status (GstSubdevSrc *subdevsrc,
    GstCameraFocusStatus *fs, gboolean detailed);

gboolean
gst_subdevsrc_libomap3camd_get_image_tags (GstSubdevSrc *subdevsrc,
    GstTagList *tlist);

gboolean
gst_subdevsrc_libomap3camd_configure (GstSubdevSrc *subdevsrc, guint *width,
    guint *height, guint32 fourcc, guint32 *size, guint *num_buffers,
    GstOperationMode mode);

gboolean
gst_subdevsrc_libomap3camd_add_buffer (GstSubdevSrc *subdevsrc,
    GstRecycleBuffer *buf, GstOperationMode mode);

gboolean
gst_subdevsrc_libomap3camd_set_vf_mode (GstSubdevSrc *subdevsrc,
    GstCameraSrcViewfinderMode mode);

gboolean
gst_subdevsrc_libomap3camd_set_autofocus (GstSubdevSrc *subdevsrc, gboolean on);

gboolean
gst_subdevsrc_libomap3camd_set_autoexposure (GstSubdevSrc *subdevsrc,
    gboolean on);

gboolean
gst_subdevsrc_libomap3camd_set_flicker_mode (GstSubdevSrc *subdevsrc,
    GstFlickerReductionMode mode);

gboolean
gst_subdevsrc_libomap3camd_read_settings (GstSubdevSrc *subdevsrc,
    GstPhotoSettings *photoconf);

gboolean
gst_subdevsrc_libomap3camd_write_settings (GstSubdevSrc *subdevsrc,
    GstPhotoSettings *photoconf, gboolean scene_override);

gboolean
gst_subdevsrc_libomap3camd_set_privacy_light (GstSubdevSrc *subdevsrc,
    gboolean on);

gboolean
gst_subdevsrc_libomap3camd_set_assist_light (GstSubdevSrc *subdevsrc,
    gboolean on);

G_END_DECLS

#endif /* __LIBOMAP3CAMD_CALLS__ */
