/* GStreamer MediaController camera source
 * Copyright (C) 2010 Nokia Corporation <multimedia@maemo.org>
 *
 * Contact: Maemo Multimedia <multimedia@maemo.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GST_MEDIACONTROLLER_H__
#define __GST_MEDIACONTROLLER_H__

#include <linux/videodev2.h>
#include <linux/media.h>
#include <linux/v4l2-subdev.h>

#include <glib.h>

typedef struct _GstMediaController GstMediaController;
typedef struct _GstMediaEntity GstMediaEntity;


/**
 * GstMediaController:
 * @entities: GList of #GstMediaEntity objects.
 * @entities_count: Number of entities found.
 * @output_size: Size of the output buffers.
 * @fd: Media controller device file descriptor.
 *
 * Contains information about media controller.
 */
struct _GstMediaController {
  GList *entities;
  int entities_count;
  guint output_size;
  int fd;
};


/**
 * GstMediaEntity:
 * @info: media_user_entity structure.
 * @pads: table of media_user_pad structures.
 * @links: table of media_user_link structures.
 * @devname: Name of the device.
 * @fd: device file descriptor.
 *
 * Contains information about media entity.
 */
struct _GstMediaEntity {
  struct media_entity_desc info;
  struct media_pad_desc *pads;
  struct media_link_desc *links;
  gchar *devname;
  int fd;
};

GstMediaController *  gst_subdevsrc_mc_init (void);

void                  gst_subdevsrc_mc_deinit (GstMediaController *mc);

int                   gst_subdevsrc_mc_get_device_fd (GstMediaController *mc,
                                                      const gchar *ename);

gboolean              gst_subdevsrc_mc_setup_pipeline (GstMediaController *mc,
                                                       const gchar *pipeline,
                                                       struct v4l2_mbus_framefmt *format);

gboolean              gst_subdevsrc_mc_link_entities (GstMediaController *mc,
                                                      const gchar *srcname,
                                                      const gchar *sinkname,
                                                      struct v4l2_mbus_framefmt *format);

gboolean              gst_subdevsrc_mc_reset_links (GstMediaController *mc);

const gchar *         gst_subdevsrc_mc_get_device_identifier (GstMediaController *mc,
                                                              int fd);

guint32               gst_subdevsrc_mc_map_pixfmt (const gchar *pixfmt);

guint32               gst_subdevsrc_mc_pcode_to_v4l2_format (enum v4l2_mbus_pixelcode code);

#endif /* __GST_MEDIACONTROLLER_H__ */
