/* GStreamer
 *
 * Copyright (C) 2010 Nokia Corporation <multimedia@maemo.org>
 *
 * subdev_calls.h - subdev system calls
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __SUBDEV_CALLS_H__
#define __SUBDEV_CALLS_H__

#include <gst/gst.h>

#include "gstsubdevcamsrc.h"

/**
 * GST_SUBDEVSRC_MAX_BUFFERS:
 *
 * Maximum amount of buffers to be used in viewfinder.
 */
#define GST_SUBDEVSRC_MAX_BUFFERS 16

/**
 * GST_SUBDEVSRC_MIN_BUFFERS:
 *
 * Minimum amount of buffers to be used in viewfinder.
 */
#define GST_SUBDEVSRC_MIN_BUFFERS 1

/**
 * GST_SUBDEVSRC_DEFAULT_BUFFERS:
 *
 * Default number of buffers to be used in viewfinder.
 */
#define GST_SUBDEVSRC_DEFAULT_BUFFERS 4


gboolean
gst_subdevsrc_open_devices (GstSubdevSrc *subdevsrc);

gboolean
gst_subdevsrc_close_devices (GstSubdevSrc *subdevsrc);

gboolean
gst_subdevsrc_start_streaming (GstSubdevSrc *subdevsrc, GstCaps *caps);

GstFlowReturn
gst_subdevsrc_grab_vf_frame (GstSubdevSrc *subdevsrc, GstBuffer **buf);

gboolean
gst_subdevsrc_stop_streaming (GstSubdevSrc *subdevsrc);

gboolean
gst_subdevsrc_update_cropping (GstSubdevSrc *subdevsrc, guint width,
    guint height, gfloat zoom);

GstCaps *
gst_subdevsrc_probe_caps (GstSubdevSrc *subdevsrc, gboolean night_mode);

gboolean
gst_subdevsrc_configure_pipeline (GstSubdevSrc *subdevsrc, guint32 *fourcc,
    guint32 *width, guint32 *height, guint32 *fps_n, guint32 *fps_d);

#endif /* __SUBDEV_CALLS_H__ */
